-- Author: B. Pankiewicz
-- This code can be used free and without any warranty and only if this comment is not removed
-- RS232 receiver with speed selected through SW_i(3)-SW_i(0) switches
-- SIGNALS description:
-- RXD_i - rs232 input signal
-- clk_i - 50MHz input clock
-- rst_i - asynchrons reset
-- data_o - output data
-- status_o outputs descibes readed in bitrate at [SW_i] according to below table:
-- | LED display |  bitrate  |   SW_i  |
-- |      F      | 230400bps | 1 1 1 1 |
-- |      E      | 115200bps | 1 1 1 0 |
-- |      D      |  57600bps | 1 1 0 1 |
-- |      C      |  38400bps | 1 1 0 0 |
-- |      B      |  28800bps | 1 0 1 1 |
-- |      A      |  19200bps | 1 0 1 0 |
-- |      9      |  14400bps | 1 0 0 1 |
-- |      8      |   9600bps | 1 0 0 0 |
-- |      7      |   4800bps | 0 1 1 1 |
-- |      6      |   2400bps | 0 1 1 0 |
-- |      5      |   1200bps | 0 1 0 1 |
-- |      4      |    300bps | 0 1 0 0 |
-- |      3      |  not used | 0 0 1 1 |
-- |      2      |  not used | 0 0 1 0 |
-- |      1      |  not used | 0 0 0 1 |
-- |      0      |  not used | 0 0 0 0 |

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity rx232 is
    Port ( clk_i : in  STD_LOGIC;
           rst_i : in  STD_LOGIC;
			  RXD_i : in  STD_LOGIC;
			  SW_i  : in STD_LOGIC_VECTOR(3 downto 0);
           data_o : out  STD_LOGIC_VECTOR (7 downto 0);
           status_o : out  STD_LOGIC_VECTOR (3 downto 0));
end rx232;

architecture Behavioral of rx232 is

	constant max_div_factor: integer:=41667;
	signal div_factor: integer range 0 to max_div_factor;
	signal enable :std_logic;
	signal main_counter: integer range 0 to 63;
	signal data_int :STD_LOGIC_VECTOR (7 downto 0);

begin
-- process reading SW3-0 and setting correct div_factor needed for speed RXD_i selection, 4 times oversampling
process(clk_i, rst_i)
begin
	if rst_i='1' then
		div_factor <= max_div_factor;
		status_o <= X"0";
	elsif Rising_Edge(clk_i) then
		case SW_i is
			when "1111" => 	div_factor<=50_000_000/230400/4; 
									status_o <= X"F";
			when "1110" => 	div_factor<=50_000_000/115200/4; 
									status_o <= X"E";
			when "1101" => 	div_factor<=50_000_000/57600/4; 
									status_o <= X"D";
			when "1100" => 	div_factor<=50_000_000/38400/4;
									status_o <= X"C";
			when "1011" => 	div_factor<=50_000_000/28800/4;
									status_o <= X"B";
			when "1010" => 	div_factor<=50_000_000/19200/4;
									status_o <= X"A";
			when "1001" => 	div_factor<=50_000_000/14400/4;
									status_o <= X"9";
			when "1000" => 	div_factor<=50_000_000/9600/4;
									status_o <= X"8";
			when "0111" => 	div_factor<=50_000_000/4800/4;
									status_o <= X"7";
			when "0110" => 	div_factor<=50_000_000/2400/4;
									status_o <= X"6";
			when "0101" => 	div_factor<=50_000_000/1200/4;
									status_o <= X"5";
			when "0100" => 	div_factor<=50_000_000/300/4;
									status_o <= X"4";
			when "0011" => 	div_factor<=50_000_000/300/4; -- not used
									status_o <= X"3";
			when "0010" => 	div_factor<=50_000_000/300/4; -- not used
									status_o <= X"2";
			when "0001" => 	div_factor<=50_000_000/300/4; -- not used
									status_o <= X"1";
			when "0000" => 	div_factor<=4; -- not used
									status_o <= X"0";
			when others => div_factor<=50_000_000/230400/4;
		end case;
	end if;
end process;


-- process generating enable with abowe set divide factor
process(clk_i, rst_i)
	variable cnt: integer range 0 to max_div_factor-1;
begin
	if rst_i='1' then
		enable <='0';
		cnt:=0;
	elsif Rising_Edge(clk_i) then
		if cnt = div_factor-1 then
			cnt := 0;
			enable <= '1';
		else
			cnt := cnt+1;
			enable <= '0';
		end if;
	end if;
end process;
	
-- main process reading RXD_i input, 4 times oversampling in respect to input speed
process(clk_i, rst_i)
begin
	if rst_i='1' then
		data_o <= (others =>'0');
		data_int <= (others =>'0');
		main_counter <= 0;
	elsif Rising_Edge(clk_i) then
		if enable = '1' then
			if (main_counter=0 and RXD_i='1') then
				main_counter <= 0;
			else
				if main_counter=39 then
					main_counter <= 0;
				else
					main_counter <= main_counter+1;
				end if;
			end if;
			case main_counter is
				when  1|2 => if RXD_i='1' then main_counter<=0;end if;
				when  6 => data_int(0) <= RXD_i;
				when 10 => data_int(1) <= RXD_i;
				when 14 => data_int(2) <= RXD_i;
				when 18 => data_int(3) <= RXD_i;
				when 22 => data_int(4) <= RXD_i;
				when 26 => data_int(5) <= RXD_i;
				when 30 => data_int(6) <= RXD_i;
				when 34 => data_int(7) <= RXD_i;
				when 38 => if RXD_i='1' then data_o <= data_int; end if;
				when others => null;
			end case;
		end if;
	end if;
end process;

end Behavioral;